﻿// -*- C++ -*-
/*!
* @file  KinectFace.h
* @brief Image on Kinect
* @date  $Date$
*
* @author 名城大学
* ロボットシステムデザイン研究室
*
* $Id$
*/

#ifndef KINECTFACE_H
#define KINECTFACE_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "CameraCommonInterfaceStub.h"
#include "BasicDataTypeStub.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="port_stub_h">
using namespace Img;
// </rtc-template>

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

using namespace RTC;

/*!
* @class KinectFace
* @brief Image on Kinect
*
* Kinectから画像を取得する,顔に関する情報を取得する
*
* Kinectの画像データ、顔方向、顔情報
*
*/
class KinectFace
	: public RTC::DataFlowComponentBase
{
public:
	/*!
	* @brief constructor
	* @param manager Maneger Object
	*/
	KinectFace(RTC::Manager* manager);

	/*!
	* @brief destructor
	*/
	~KinectFace();

	// <rtc-template block="public_attribute">

	// </rtc-template>

	// <rtc-template block="public_operation">

	// </rtc-template>

	/***
	* Kinectへ接続する
	*
	* The initialize action (on CREATED->ALIVE transition)
	* formaer rtc_init_entry()
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	virtual RTC::ReturnCode_t onInitialize();

	/***
	*
	*
	* The finalize action (on ALIVE->END transition)
	* formaer rtc_exiting_entry()
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	// virtual RTC::ReturnCode_t onFinalize();

	/***
	*
	* The startup action when ExecutionContext startup
	* former rtc_starting_entry()
	*
	* @param ec_id target ExecutionContext Id
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	// virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

	/***
	*
	* The shutdown action when ExecutionContext stop
	* former rtc_stopping_entry()
	*
	* @param ec_id target ExecutionContext Id
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	// virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

	/***
	* ポート及びデータの初期化
	*
	* The activated action (Active state entry action)
	* former rtc_active_entry()
	*
	* @param ec_id target ExecutionContext Id
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

	/***
	* データの破棄
	*
	* The deactivated action (Active state exit action)
	* former rtc_active_exit()
	*
	* @param ec_id target ExecutionContext Id
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

	/***
	* 画像データ出力
	*
	* The execution action that is invoked periodically
	* former rtc_active_do()
	*
	* @param ec_id target ExecutionContext Id
	*
	* @return RTC::ReturnCode_t
	*
	* @pre Kinectに接続できていること
	*
	*/
	virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

	/***
	*
	* The aborting action when main logic error occurred.
	* former rtc_aborting_entry()
	*
	* @param ec_id target ExecutionContext Id
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	// virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

	/***
	*
	* The error action in ERROR state
	* former rtc_error_do()
	*
	* @param ec_id target ExecutionContext Id
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	// virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

	/***
	* Kinectへの再接続を試みる
	*
	* The reset action that is invoked resetting
	* This is same but different the former rtc_init_entry()
	*
	* @param ec_id target ExecutionContext Id
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

	/***
	*
	* The state update action that is invoked after onExecute() action
	* no corresponding operation exists in OpenRTm-aist-0.2.0
	*
	* @param ec_id target ExecutionContext Id
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	// virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

	/***
	*
	* The action that is invoked when execution context's rate is changed
	* no corresponding operation exists in OpenRTm-aist-0.2.0
	*
	* @param ec_id target ExecutionContext Id
	*
	* @return RTC::ReturnCode_t
	*
	*
	*/
	// virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);
	void KinectFace::drawColorFrame();
	void KinectFace::drawDepthFrame();
	void KinectFace::updateBodyFrame();
	void KinectFace::updateFaceFrame();

protected:
	// <rtc-template block="protected_attribute">

	// </rtc-template>

	// <rtc-template block="protected_operation">

	// </rtc-template>

	// Configuration variable declaration
	// <rtc-template block="config_declare">
	/*!
	*
	* - Name:  Output
	* - DefaultValue: raw,camera,jpeg
	*/
	std::string m_Output;
	/*!
	*
	* - Name:  Size
	* - DefaultValue: 480x270
	*/
	std::string m_Size;

	// </rtc-template>

	// DataInPort declaration
	// <rtc-template block="inport_declare">

	// </rtc-template>


	// DataOutPort declaration
	// <rtc-template block="outport_declare">
	Img::TimedCameraImage m_CameraImage;
	/*!
	* CameraImage型の生データ
	* - Type: CameraImage
	* - Number: 1
	*/
	OutPort<Img::TimedCameraImage> m_CameraImageOut;
	RTC::TimedDoubleSeq m_FaceAngle;
	/*!
	* 顔のクォータニオン角度
	* - Number: 3
	*/
	OutPort<RTC::TimedDoubleSeq> m_FaceAngleOut;
	RTC::TimedBooleanSeq m_Faceproperty;
	/*!
	* 顔の情報　笑顔,右目開いてるか,左目開いてるか等等
	*/
	OutPort<RTC::TimedBooleanSeq> m_FacepropertyOut;

	// </rtc-template>

	// CORBA Port declaration
	// <rtc-template block="corbaport_declare">

	// </rtc-template>

	// Service declaration
	// <rtc-template block="service_declare">

	// </rtc-template>

	// Consumer declaration
	// <rtc-template block="consumer_declare">

	// </rtc-template>

private:
	// <rtc-template block="private_attribute">

	// </rtc-template>

	// <rtc-template block="private_operation">

	// </rtc-template>

};

template <class T> void SafeRelease(T *&ppT) {
	if (ppT) {
		ppT->Release();
		ppT = NULL;
	}
}


extern "C"
{
	DLL_EXPORT void KinectFaceInit(RTC::Manager* manager);
};

#endif // KINECTFACE_H
