﻿// -*- C++ -*-
// -*- coding: utf-8-ws -*-


#ifndef TROBOT_UTIL_HPP
#define TROBOT_UTIL_HPP

#include "TRobotConstants.hpp"
#include "TRobotSensorData.hpp"
#include <cstddef>

namespace TRobot
{
  RetCode buildCmdSeqSetRotVel(int leftRotVel, int rightRotVel, unsigned char *cmd);
  bool isValidHeader(const unsigned char * const header);
  RetCode decodeBaseSensorData(const unsigned char * const buf, BaseSensorData& sensorData);
  
  const unsigned char *searchForValidHeader(const unsigned char *bufFirst,
                                            const unsigned char *bufLast);

  template<class T> inline const T& limitRange
  (const T& val, const T& minVal, const T& maxVal)
  {
    return std::max(std::min(val, maxVal), minVal);
  }  


  class RotVelCmdPair : std::pair<int, int>
  {
  public:
    RotVelCmdPair(int left = 0, int right = 0) : std::pair<int, int>(left, right) {}
    inline int left()  { return this->first; } 
    inline int right() { return this->second; }
  };

  const size_t BASE_SENSOR_DATA_LEN = 145; // [bytes]
  const size_t BASE_SENSOR_DATA_HEADER_LEN = 3;
  const int ROT_VEL_CMD_MIN = -100;
  const int ROT_VEL_CMD_MAX = +100;
  const size_t ROT_VEL_CMD_LEN = 8;
  const int CTRL_RATE = 16;
  const int SEC_TO_USEC = 1000*1000;
  const int CTRL_PERIOD = SEC_TO_USEC/CTRL_RATE; // [usec]
}

#define DBGOUT(line) { std::cerr line; }

#endif // TROBOT_UTIL_HPP




