﻿#include "TRobotSerialIOIface.hpp"
#include "TRobotSerialIOLinux.hpp"
#include "TRobotSensorData.hpp"
#include <iostream>
#include <cstdlib>
#include <unistd.h>

int main(int argc, char *argv[])
{
  using namespace std;
  TRobot::SerialIOIface *serIO = new TRobot::SerialIOLinux();
  const int CTRL_RATE = 16;
  const int SEC_TO_USEC = 1000 * 1000;
  const useconds_t PERIOD = SEC_TO_USEC/CTRL_RATE; // 10*1000; 

  TRobot::BaseSensorData sdata;
  TRobot::RetCode retVal;
  int leftRotVel  = 0;
  int rightRotVel = 0;
  unsigned long numIterations = 0;

#ifdef CFG_TROBOT2
  retVal = serIO->start("/dev/ttyUSB0");
#else // elif defined CFG_TROBOT2
  if(argc > 1){
    retVal = serIO->start(argv[1]);
  }else{
    retVal = serIO->start("/dev/ttyACM0");
  }
#endif
  if(retVal != TRobot::SUCCESS){
    exit(EXIT_FAILURE);
  }
  cout << "input vel (e.g. 50 50): " << endl;
  cin >> leftRotVel >> rightRotVel;
  cout << "lrv = " << leftRotVel << " rrv = " << rightRotVel << endl;
  //serIO->sendWheelRotVel(leftRotVel, rightRotVel);
  
  while(true){
    serIO->sendWheelRotVel(leftRotVel, rightRotVel);
    retVal = serIO->receiveSensorData(sdata);
    cerr << "numIterations == " << numIterations << endl;
    numIterations++;
    
    if(retVal == TRobot::SUCCESS){
      for(size_t i = 0; i < TRobot::BaseSensorData::NUM_ENC_COUNTS; i++){
        cout << sdata.m_leftEnc[i] << " " << sdata.m_rightEnc[i] << " "
             << sdata.m_cycleCount[i] << " ";
      }
      for(size_t i = 0; i < TRobot::BaseSensorData::NUM_XYZ; i++){
        cout << sdata.m_acc[i] << " ";
      }
      for(size_t i = 0; i < TRobot::BaseSensorData::NUM_XYZ; i++){
        cout << sdata.m_gyr[i] << " ";
      }
      for(size_t i = 0; i < TRobot::BaseSensorData::NUM_XYZ; i++){
        cout << sdata.m_mag[i] << " ";
      }
      cout << " END" << endl;
    }else{
      cerr << "Error on receiving data" << endl;
    }
    usleep(PERIOD);
  }

  retVal = serIO->stop();
  delete serIO;
}
