#!/usr/bin/env python
# -*- coding: utf-8 -*- 
# -*- Python -*-


import sys
sys.path.append(".")

import RTC
import OpenRTM_aist

import numpy as np

# _DEFAULT_WALL_ST_ANG  = -90.0,
# _DEFAULT_WALL_ED_ANG  = -30.0,
_DEFAULT_FRONT_ST_ANG = -25.0 # 前方に相当する角度範囲の始点 [deg]
_DEFAULT_FRONT_ED_ANG =  25.0 # 前方に相当する角度範囲の終点 [deg]


# This module's spesification
# <rtc-template block="module_spec">
wallguidedpathplanner_spec = [
  "implementation_id", "WallGuidedPlanner", 
  "type_name",         "WallGuidedPlanner", 
  "description",       "Wall-guided motion planner component", 
  "version",           "0.1", 
  "vendor",            "TIRI", 
  "category",          "Motion planner", 
  "activity_type",     "DataFlowComponent", 
  "max_instance",      "10", 
  "language",          "Python", 
  "lang_type",         "SCRIPT",
  # "conf.default.wallStAng",  str(_DEFAULT_WALL_ST_ANG), 
  # "conf.default.wallEdAng",  str(_DEFAULT_WALL_ED_ANG), 
  "conf.default.frontStAng", str(_DEFAULT_FRONT_ST_ANG),
  "conf.default.frontEdAng", str(_DEFAULT_FRONT_ED_ANG),
  ""]

# </rtc-template>

# global _DEG2RAD
# global _DIST_ERR, DIST_ERR_OUT

_DEG2RAD = np.pi/180.0 
_DIST_ERR = 0.000001 # 検出失敗だと ranges[i] == 0.0
_DIST_ERR_OUT = 1.0 #

#!  [rad] -> [deg]
def toRadians(degAng):
  return degAng*_DEG2RAD


# //////////////////////////////////////////////////////////////////////////////
#!
# 壁沿い運動計画を行う RTコンポーネント
#
#
class WallGuidedPlanner(OpenRTM_aist.DataFlowComponentBase):
  def __init__(self, manager):
    OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

    self.ranges = []
    self.rangeStAng = -np.pi*2.0/3.0 # [rad], URG-04LXの仕様
    self.rangeEdAng = +np.pi*2.0/3.0 # [rad], URG-04LXの仕様
    self.rangeAngRes = 2*np.pi/1024  # [rad], URG-04LXの仕様
    
    # config. data
    # self._wallStAng  = [_DEFAULT_WALL_ST_ANG]  
    # self._wallEdAng  = [_DEFAULT_WALL_ED_ANG]  
    self._frontStAng = [_DEFAULT_FRONT_ST_ANG]  
    self._frontEdAng = [_DEFAULT_FRONT_ED_ANG]  
    
  def onInitialize(self):
    # ポートの準備
    _pose3D = RTC.Pose3D(RTC.Point3D(0.0, 0.0, 0.0),
                         RTC.Orientation3D(0.0, 0.0, 0.0))
    _size3D = RTC.Size3D(0.0, 0.0, 0.0)
    _geometry3D = RTC.Geometry3D(_pose3D, _size3D)
    _rangerConfig = RTC.RangerConfig(0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0)
    
    self._d_rangeData = RTC.RangeData(RTC.Time(0,0),
                                      [],
                                      RTC.RangerGeometry(_geometry3D, []),
                                      _rangerConfig)
    
    self._rangeDataIn = OpenRTM_aist.InPort("rangeData", self._d_rangeData)
    self.addInPort("rangeData", self._rangeDataIn)
    

    self._d_refVel = RTC.TimedVelocity2D(RTC.Time(0,0),
                                         RTC.Velocity2D(0, 0, 0))
    self._refVelOut = OpenRTM_aist.OutPort("refVel", self._d_refVel)
    self.addOutPort("refVel", self._refVelOut)

    
    # self.bindParameter("wallStAng", self._wallStAng, str(_DEFAULT_WALL_ST_ANG))
    # self.bindParameter("wallEdAng", self._wallEdAng, str(_DEFAULT_WALL_ED_ANG))
    self.bindParameter("frontStAng", self._frontStAng, str(_DEFAULT_FRONT_ST_ANG))
    self.bindParameter("frontEdAng", self._frontEdAng, str(_DEFAULT_FRONT_ED_ANG))
    
    return RTC.RTC_OK


  #def onFinalize(self):
  #  return RTC.RTC_OK

  #def onStartup(self, ec_id):
  #  return RTC.RTC_OK

  #def onShutdown(self, ec_id):
  #  return RTC.RTC_OK

  def onActivated(self, ec_id):
    return RTC.RTC_OK

  def onDeactivated(self, ec_id):
    self.ranges = []

    return RTC.RTC_OK

  #!
  #  @param ranges LRF のデータ
  def planVelocity(self, ranges):
    if len(ranges) == 0:
      return (0.0, 0.0, 0.0)
    
    THRESH_DIST = 0.4 # [m]
#    VA_OBST_AVOID = -0.2 # [rad/s]
    VA_OBST_AVOID = -0.08
    TAN_VEL = 0.00 # [m/s], 軌道の接線方向の速度目標値 # ボディの都合で逆向きにする場合
    #TAN_VEL = 0.08 # [m/s], 軌道の接線方向の速度目標値
    
    # print "Converting angles..."
    th_fs = toRadians(self._frontStAng[0])
    th_fe = toRadians(self._frontEdAng[0])

    r = ranges
    th = np.linspace(self.rangeStAng, self.rangeEdAng, num=r.shape[0])
    inlierIdx = (_DIST_ERR < r) * (r < _DIST_ERR_OUT)
    r  = r[inlierIdx]
    th = th[inlierIdx]
    # print "r.shape = ", r.shape, ", th.shape = ", th.shape
    
    # px = r*np.cos(th)
    # py = r*np.sin(th)
    #cpos = np.vstack([px, py]).T


    print "Checkig obstacles in front..."
    # 前方に障害物（壁）があるか否かを調べる
    rf = r[(th_fs <= th) * (th <= th_fe)]
    if np.sum((_DIST_ERR < rf) *(rf < THRESH_DIST)) > 0:
      print "Obstacles found in front"
      # 障害物回避を優先する
      return (0, 0, VA_OBST_AVOID)

    else:
      return (TAN_VEL, 0, 0)

  
  def onExecute(self, ec_id):
    if self._rangeDataIn.isNew():
      _rangeData = self._rangeDataIn.read()
      self.ranges = np.array(_rangeData.ranges)
      self.rangeStAng  = _rangeData.config.minAngle
      self.rangeEdAng  = _rangeData.config.maxAngle
      self.rangeAngRes = _rangeData.config.angularRes

    # print self.ranges
    print "Planning..." 
    vx, vy, va = self.planVelocity(self.ranges)

    self._d_refVel.data.vx = vx
    self._d_refVel.data.vy = vy
    self._d_refVel.data.va = va
    print "Changed refVel: ", [vx, vy, va]

    OpenRTM_aist.setTimestamp(self._d_refVel)
    self._refVelOut.write()
      
    return RTC.RTC_OK


  

  #def onAborting(self, ec_id):
  #  return RTC.RTC_OK

  #def onError(self, ec_id):
  #  return RTC.RTC_OK

  #def onReset(self, ec_id):
  #  return RTC.RTC_OK

  #def onStateUpdate(self, ec_id):
  #  return RTC.RTC_OK

  #def onRateChanged(self, ec_id):
  #  return RTC.RTC_OK



def WallGuidedPlannerInit(manager):
  profile = OpenRTM_aist.Properties(defaults_str=wallguidedpathplanner_spec)
  manager.registerFactory(profile,
                          WallGuidedPlanner,
                          OpenRTM_aist.Delete)


def MyModuleInit(manager):
  WallGuidedPlannerInit(manager)

  # Create a component
  comp = manager.createComponent("WallGuidedPlanner")



def main():
  mgr = OpenRTM_aist.Manager.init(sys.argv)
  mgr.setModuleInitProc(MyModuleInit)
  mgr.activateManager()
  mgr.runManager()

if __name__ == "__main__":
  main()
  
